// Ausstattung.java                                 MM 2014
/**
* Ausstattung eines Mietautos,
* bestehend aus 5 Ausstattungsmerkmalen.
* Berechnen des Gesamtmietzuschlag
* entsprechend den vorhandenen Ausstattungsmerkmalen.
*/
public class Ausstattung
{
  /**
  * Merkmal Automatik.
  */
  private boolean automatik = false;
  
  /**
  * Merkmal Klimaanlage.
  */
  private boolean klima = false;
  
  /**
  * Merkmal Navigationssystem.
  */
  private boolean navi = false;
  
  /**
  * Merkmal Parksensoren.
  */
  private boolean park = false;
  
  /**
  * Merkmal Winterreifen.
  */
  private boolean winter = false;
  
  /**
  * Besonderheiten zur Ausstattung.
  */
  private String sonst = "";
  
  /**
  * Mietzuschlag pro Ausstattungsmerkmal in Euro.
  */
  private static int zuschlag = 5;
  
/* ----------------------------------------------------- */

  /**
  * Konstruktor, setzt die Ausstattungsmerkmale.
  * @param automatik true, falls Automatik vorhanden
  * @param klima     true, falls Klimaanlage vorhanden
  * @param navi      true, falls Navigationssystem vorhanden
  * @param park      true, falls Parksensoren vorhanden
  * @param winter    true, falls Winterreifen vorhanden
  */
  public Ausstattung
  ( boolean automatik, boolean klima, boolean navi,
    boolean park, boolean winter )
  {
    this.automatik = automatik;
    this.klima     = klima;
    this.navi      = navi;
    this.park      = park;
    this.winter    = winter;
  }
  
/* ----------------------------------------------------- */

  /**
  * Setzt den Mietzuschlag pro vorhandenem
  * Ausstattungsmerkmal.
  * @param euro Zuschlag in Euro
  */
  public static void setZuschlag( int euro )
  {
    zuschlag = euro;
  }
  
  /**
  * Setzt Besonderheiten zur Ausstattung.
  * @param sonst Besonderheiten
  */
  public void setSonst( String sonst )
  {
    this.sonst = sonst;
  }
  
  /**
  * Reifen werden neu aufgezogen.
  * @param winter true,  falls es Winterreifen sind
  *               false, falls es Sommerreifen sind
  */
  public void reifenWechsel( boolean winter )
  {
    this.winter = winter;
  }
  
  /**
  * Berechnet Gesamtmietzuschlag
  * entsprechend den vorhandenen Ausstattungsmerkmalen.
  * @return Gesamtmietzuschlag in Euro
  */
  public int berechneGesamtZuschlag()
  {
    int z = 0;
    if( automatik) z++;
    if( klima)     z++;
    if( navi)      z++;
    if( park)      z++;
    if( winter)    z++;
    return z * zuschlag;
  }
  
  /**
  * Gibt als String zurueck:
  * Vorhandene Ausstattungsmerkmale,
  * Besonderheiten und Gesamtmietzuschlag.
  * @return Ausstattungsbeschreibung
  */
  public String toString()
  {
    String str = "";
    if( automatik) str += "Automatik ";
    if( klima)     str += "Klimaanlage ";
    if( navi)      str += "Navigationssystem ";
    if( park)      str += "Parksensoren ";
    if( winter)    str += "Winterausruestung ";
    if( !sonst.equals("")) str += "\nBesonderheiten: " + sonst;
    str += "\nZuschlag: " + berechneGesamtZuschlag() + " Euro";
    return str;
  }

/* ----------------------------------------------------- */

  /**
  * Testprogramm:
  * Erzeugt Ausstattungen von 6 Testautos,
  * fuehrt einen Reifenwechsel durch und
  * gibt die Ausstattungen auf der Konsole aus.
  */
  public static void main( String[] args )
  {
    Ausstattung ausstattung;
    
    ausstattung
    = new Ausstattung( true, true, true, false, false);
    ausstattung.setSonst( "neu"); 
    System.out.println( "\nAusstattung:\n" + ausstattung);

    ausstattung
    = new Ausstattung( false, true, false, false, false);
    ausstattung.setSonst( "Multimedia"); 
    System.out.println( "\nAusstattung:\n" + ausstattung);

    ausstattung
    = new Ausstattung( false, true, true, true, false);
    System.out.println( "\nAusstattung:\n" + ausstattung);

    ausstattung
    = new Ausstattung( true, true, true, false, false);
    System.out.println( "\nAusstattung:\n" + ausstattung);

    ausstattung
    = new Ausstattung( false, true, true, false, false);
    ausstattung.setSonst( "Cabrio"); 
    System.out.println( "\nAusstattung:\n" + ausstattung);

    ausstattung
    = new Ausstattung( true, true, true, true, false);
//    System.out.println( "\nAusstattung:\n" + ausstattung);
    ausstattung.reifenWechsel( true); 
    System.out.println
    ( "\nAusstattung nach Reifenwechsel:\n" + ausstattung);
  }
}