// Fuhrpark.java                                    MM 2014
import Tools.IO.*;                              // Eingaben
import java.util.*;                           // LinkedList

/**
* Verwaltung eines Fuhrparks einer Autovermietung.
*/
public class Fuhrpark
{
  /**
  * Alle zur Vermietung freien Autos befinden sich
  * auf dem Parkplatz.
  */
  private Parkplatz parkplatz;
  
  /**
  * Alle vermieteten Autos befinden sich auf der Strasse.
  */
  private Strasse strasse;
  
  /**
  * Alle zurueckgegebenen Autos werden in der Werkstatt
  * gewartet.
  */
  private Werkstatt werkstatt;
  
/* ----------------------------------------------------- */

  /**
  * Konstruktor,
  * richtet Parkplatz, Strasse und Werkstatt ein.
  */
  public Fuhrpark()
  {
    parkplatz = new Parkplatz();
    strasse   = new Strasse();
    werkstatt = new Werkstatt();
  }
  
/* ----------------------------------------------------- */

  /**
  * Verwaltungsprogramm:
  * Richtet Fuhrpark ein,
  * ermoeglicht die Ausfuehrung von Auftraegen und
  * ermoeglicht die Anzeige der Autos an den Orten.
  * Verwendet Menue "Anzeige Auftrag"
  * und Menue "Anzeige Ort".
  */
  public static void main( String[] args )
  {
    Fuhrpark fuhrpark = new Fuhrpark();
    
    System.out.println
    ( "\nVerwaltungsmenue einer Autovermietung\n");
    
    int menueAuswahl = 0;
    do
    {
      //Auswahl und Durchfuehrung der Auftraege
      menueAuswahl = fuhrpark.menueAnzeigeAuftrag();
      if( menueAuswahl == -1) break;
      
      //Auswahl und Anzeige der Orte
      menueAuswahl = fuhrpark.menueAnzeigeOrt();
      if( menueAuswahl == -1) break;
    } while( true);
  }
  
/* ----------------------------------------------------- */

  /**
  * Menue "Anzeige Ort",
  * ermoeglicht die Auswahl eines Ortes und
  * zeigt die Autos vor Ort auf der Konsole an.
  * Orte: "Parkplatz", "Strasse", "Werkstatt".
  * @return  0 weiter
  *         -1 Programm beenden
  */
  public int menueAnzeigeOrt()
  {
    int menueAuswahl = 0;
    do
    {
      // Menue Auswahl eines Ortes
      System.out.println( "\nAutoaufenthaltsorte:");
      System.out.println( "\t 1 .. Parkplatz");
      System.out.println( "\t 2 .. Strasse");
      System.out.println( "\t 3 .. Werkstatt");
      System.out.println( "\n\t 0 .. weiter");
      System.out.println( "\t-1 .. Programm beenden");
      menueAuswahl = IOTools.readInteger
        ( "Autoaufenthaltsort auswaehlen: ");

      // Anzeige der Autos vor Ort
      switch( menueAuswahl)
      {
        case 1: System.out.println
                ( "\nParkplatz:\n" + parkplatz); break;
        case 2: System.out.println
                ( "\nStrasse:\n" + strasse); break;
        case 3: System.out.println
                ( "\nWerkstatt:\n" + werkstatt); break;
      }
    } while( menueAuswahl > 0);
    return menueAuswahl;
  }

  /**
  * Menue "Anzeige Auftrag",
  * ermoeglicht die Auswahl eines Auftrages und
  * veranlasst dessen Ausfuehrung:
  * "Neues Auto"             : auftragNeuesAuto()
  * "Vermieten"              : auftragVermieten()
  * "Strasse -> Werkstatt"   : auftragWerkstatt()
  * "Werkstatt -> Parkplatz" : auftragParkplatz()
  * "Verschrotten"           : auftragVerschrotten()
  * @return  0  weiter
  *         -1 Programm beenden
  */
  public int menueAnzeigeAuftrag()
  {
    int menueAuswahl = 0;
    do
    {
      // Menue Auswahl eines Auftrags
      System.out.println( "\nAutoaktionen:");
      System.out.println( "\t 1 .. Neues Auto");
      System.out.println( "\t 2 .. Vermieten");
      System.out.println( "\t 3 .. Strasse -> Werkstatt");
      System.out.println( "\t 4 .. Werkstatt -> Parkplatz");
      System.out.println( "\t 5 .. Verschrotten");
      System.out.println( "\n\t 0 .. weiter");
      System.out.println( "\t-1 .. Programm beenden");
      menueAuswahl = IOTools.readInteger( "Autoaktion auswaehlen: ");

      // Ausfuehrung des Auftrages
      switch( menueAuswahl)
      {
        case 1: auftragNeuesAuto(); break;
        case 2: auftragVermieten(); break;
        case 3: auftragWerkstatt(); break;
        case 4: auftragParkplatz(); break;
        case 5: auftragVerschrotten(); break;
      }
    } while( menueAuswahl > 0);
    return menueAuswahl;
  }
  
/* ----------------------------------------------------- */
// Neues Auto
  
  /**
  * Auftrag "Neues Auto",
  * ermoeglicht die Auswahl eines Autos und
  * stellt es auf den Parkplatz ab.
  * Verwendet Menue "Neues Auto".
  */
  private void auftragNeuesAuto()
  {
    Auto auto = menueNeuesAuto();
    if( auto != null) parkplatz.anmelden( auto); 
  }
  
  /**
  * Menue "Neues Auto",
  * erfasst Wagentyp, Ausstattung und Grundpreis.
  * Verwendet Menue "Ausstattung".
  * @return neues Auto
  */
  private Auto menueNeuesAuto()
  {
    // Menue Auswahl Wagentyp
    System.out.println( "\nWagentyp:");
    System.out.println( "\t 1 - Kleinwagen");
    System.out.println( "\t 2 - Mittelklassewagen");
    System.out.println( "\t 3 - Oberklassewagen");
    int menueAuswahl =
      IOTools.readInteger( "Wagentyp auswaehlen: ");

    // Ausstattung
    Ausstattung ausstattung = menueAusstattung();
    
    // Grundpreis
    System.out.println();
    int preis = 0;
    do
    {
      preis = IOTools.readInteger( "Grundpreis in Euro: ");
    } while( preis < 0);

    Auto auto = null;
    switch( menueAuswahl)
    {
      case 1: auto = new Kleinwagen( ausstattung, preis);
              break;
      case 2: auto
              = new Mittelklassewagen( ausstattung, preis);
              break;
      case 3: auto
              = new Oberklassewagen( ausstattung, preis);
              break;
      default: System.out.println( "Fehlerhafte Eingabe!");
    }

    System.out.println();
    return auto;
  }
  
  /**
  * Menue "Ausstattung",
  * erfragt die Ausstattung eines neuen Autos:
  * "Automatik" "Klimaanlage" "Navigationssystem"
  * "Parksensoren" "Winterreifen",
  * erzeugt entsprechende Ausstattung und
  * registriert Besonderheiten der Ausstattung.
  * @return Ausstattung
  */
  private Ausstattung menueAusstattung()
  {
    System.out.println( "\nAusstattung:");
    
    boolean automatik = setMerkmal( "Automatik        ");
    boolean klima     = setMerkmal( "Klimaanlage      ");
    boolean navi      = setMerkmal( "Navigationssystem");
    boolean park      = setMerkmal( "Parksensoren     ");
    boolean winter    = setMerkmal( "Winterreifen     ");

    Ausstattung ausstattung
    = new Ausstattung( automatik, klima, navi, park, winter);
    
    // Besonderheiten
    System.out.println();
    String str = "";
    str = IOTools.readLine( "Besonderheiten zur Ausstattung: ");
    ausstattung.setSonst( str); 

    return ausstattung;
  }
  
  /**
  * Fragt nach einem Ausstattungsmerkmal.
  * @param strMerkmal Merkmal als String
  * @return true, falls Merkmal vorhanden
  */
  private boolean setMerkmal( String strMerkmal )
  {
    char merkmal = ' ';
    do
    {
      merkmal = IOTools.readChar( strMerkmal + " (j/n)\t");
    } while( merkmal != 'j' && merkmal != 'n');

    if( merkmal == 'j')  return true;
    return false;
  }

/* ----------------------------------------------------- */
// Vermieten

  /**
  * Auftrag "Vermieten",
  * eine zu vermietendes Auto wird ausgewaehlt
  * und vom Parkplatz auf die Strasse geschickt.
  * Verwendet Menue "Auswahl Auto".
  */
  private void auftragVermieten()
  {
    // Auto auswaehlen
    System.out.println( "Parkplatz:");
    Auto auto = auswahlAuto( parkplatz);

    // Auto: Parkplatz -> Strasse
    if( auto != null)
    {
      parkplatz.abmelden( auto); 
      strasse.anmelden( auto); 
    }
    else
      System.out.println
      ( "Kein passendes Auto auf dem Parkplatz!");
  }

// Strasse -> Werkstatt

  /**
  * Auftrag "Strasse -> Werkstatt",
  * ein zurueckgegebenes Auto kommt zur Wartung
  * in die Werkstatt.
  * Verwendet Menue "Auswahl Auto".
  */
  private void auftragWerkstatt()
  {
    // Auto auswaehlen
    System.out.println( "Strasse:");
    Auto auto = auswahlAuto( strasse);

    // Auto: Strasse -> Werkstatt
    if( auto != null)
    {
      strasse.abmelden( auto); 
      werkstatt.anmelden( auto); 
    }
    else
      System.out.println
      ( "Kein passendes Auto auf der Strasse!");
  }
  
// Werkstatt -> Parkplatz

  /**
  * Auftrag "Werkstatt -> Parkplatz",
  * ein Auto wird von der Werkstatt auf den Parkplatz
  * zurueck zur Vermietung gefahren.
  * Verwendet Menue "Auswahl Auto".
  */
  private void auftragParkplatz()
  {
    // Auto auswaehlen
    System.out.println( "Werkstatt:");
    Auto auto = auswahlAuto( werkstatt);

    // Auto: Werkstatt -> Parkplatz
    if( auto != null)
    {
      werkstatt.abmelden( auto); 
      parkplatz.anmelden( auto); 
    }
    else
      System.out.println
      ( "Kein passendes Auto in der Werkstatt!");

  }

// Verschrotten

  /**
  * Auftrag "Verschrotten",
  * ein Auto wird verschrottet.
  * Verwendet Menue "Auswahl Auto".
  */
  private void auftragVerschrotten()
  {
    // Auto auswaehlen
    System.out.println( "Werkstatt:");
    Auto auto = auswahlAuto( werkstatt);

    // Auto verschrotten
    if( auto != null)
    {
      werkstatt.abmelden( auto);
    }
    else
      System.out.println
      ( "Kein passendes Auto in der Werkstatt!");
  }

  /**
  * Menue "Auswahl Auto",
  * alle Autos vor Ort werden angezeigt
  * und ein Auto ueber dessen Kennzeichen ausgewaehlt.
  * @param ort Ort
  * @return ausgewaehltes Auto
  *         null, falls kein Auto da oder Kennzeichen falsch
  */
  private Auto auswahlAuto( Ort ort)
  {
    // Autos vor Ort
    LinkedList<Auto> autos = ort.getAlleAutos();
    if( autos.isEmpty()) return null;     // keine Autos da
    System.out.println( ort);

    // Auswahl eines Autos
    int kennung
      = IOTools.readInteger( "\nAuto auswaehlen: L-RA-");
    for( Auto auto: autos)
      if( auto.getKennZeichen().equals( "L-RA-" + kennung))
        return auto;
    return null;                      // Kennzeichen falsch
  }
}